! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! Main program to execute elastic modeling
program Elastic_program

  use sep
  use elastic_mod_new

  use omp_lib

  implicit none

  integer                               :: nz,nx,nt,nsource,xsource,zsource,iz,ix
  integer                               :: nsnap, stat, sp, ti, abc, zrec, nseis
  real                                  :: dz,oz,dx,ox,dt,ot,fp, dtseis, Tt
  real, allocatable, dimension(:,:)     :: vp, vs, rho, seisp, seiss,lambda,mi
  real, allocatable, dimension(:,:,:)   :: snapvx,snapvz
  logical                               :: lame
  real                                  :: vmax

  !Variables to measure runtime
  real*8                                :: start_time,end_time

  start_time = omp_get_wtime()

  call sep_init()

  ! Read SEP header parameter
  ! Model parameters
  call from_history("n1",nz)
  call from_history("o1",oz)
  call from_history("d1",dz)
  call from_history("n2",nx)
  call from_history("o2",ox)
  call from_history("d2",dx)
  !call from_param("nz",nz,1000)
  !call from_param("oz",oz,0.0)
  !call from_param("dz",dz,6.0)
  !call from_param("nx",nx,1000)
  !call from_param("ox",ox,0.0)
  !call from_param("dx",dx,6.0)
  ! Simulation parameters
  call from_param("nsnap",nsnap,50)        !# of snapshots, default=50
  call from_param("xsource",xsource)       !position of source in x samples
  call from_param("zsource",zsource)       !position of source in z samples
  call from_param("Ttime",Tt)              !total propagation time
  call from_param("ot",ot,0.0)             !start of seismogram, default=0
  call from_param("fpeak",fp,25.0)         !Ricker source peak freq.
  call from_param("sp",sp)                 !order of spatial FD approx.
  call from_param("ti",ti,2)               !order of time FD, default=2
  call from_param("abc",abc,0)             !# of absorbing border samples
  call from_param("zrec",zrec,1)           !receivers depth
  call from_param("dtseis",dtseis,0.002)   !seismogram sampling, default=2ms
  call from_param("lame",lame,.false.)     !if lame=.true., lame parameters will be
                                           !used instead of vp and vs

  ! dt from the Courant stability condition => C=(vx*dt)/dx+(vz*dt)dz<1
  ! For the simple case of dz=dx and using Virieux(1986) relation:
  dt = dx/(sqrt(2.0)*4*2700.0) !the 4 is arbitrary, but it was diverging otherwise

  ! size of seismogram in time
  nseis=Tt/dtseis

  ! Write SEP header parameter
  ! seisp.H output parameters
  call to_history("n1",nseis)
  call to_history("o1",ot)
  call to_history("d1",dtseis)
  call to_history("n2",(nx-2*abc))
  call to_history("o2",ox)
  call to_history("d2",dx)
  call to_history("label1","t")
  call to_history("label2","x")
  ! seiss.H output parameters
  call to_history("n1",nseis,"seiss")
  call to_history("o1",ot,"seiss")
  call to_history("d1",dtseis,"seiss")
  call to_history("n2",(nx-2*abc),"seiss")
  call to_history("o2",ox,"seiss")
  call to_history("d2",dx,"seiss")
  call to_history("label1","t","seiss")
  call to_history("label2","x","seiss")
  ! snapshot of x velocities output parameters
  call to_history("n1",nz,"snapvx")
  call to_history("o1",oz,"snapvx")
  call to_history("d1",dz,"snapvx")
  call to_history("n2",nx,"snapvx")
  call to_history("o2",ox,"snapvx")
  call to_history("d2",dx,"snapvx")
  call to_history("n3",nsnap,"snapvx")
  call to_history("o3",0,"snapvx")
  call to_history("d3",1,"snapvx")
  call to_history("label1","z","snapvx")
  call to_history("label2","x","snapvx")
  call to_history("label3","t","snapvx")
  ! snapshot of z velocities output parameters
  call to_history("n1",nz,"snapvz")
  call to_history("o1",oz,"snapvz")
  call to_history("d1",dz,"snapvz")
  call to_history("n2",nx,"snapvz")
  call to_history("o2",ox,"snapvz")
  call to_history("d2",dx,"snapvz")
  call to_history("n3",nsnap,"snapvz")
  call to_history("o3",0,"snapvz")
  call to_history("d3",1,"snapvz")
  call to_history("label1","z","snapvz")
  call to_history("label2","x","snapvz")
  call to_history("label3","t","snapvz")

  ! Allocation of the input/output matrices
  if(lame)then
    allocate(lambda(nz,nx))
    allocate(mi(nz,nx))
  else
    allocate(vp(nz,nx))
    allocate(vs(nz,nx))
  endif
  allocate(rho(nz,nx))
  allocate(seisp(nseis,(nx-2*abc)))
  allocate(seiss(nseis,(nx-2*abc)))
  allocate(snapvx(nz,nx,nsnap))
  allocate(snapvz(nz,nx,nsnap))

  ! Initialize values
  if(lame)then
    lambda=0.0
    mi=0.0
  else
    vp=0.0
    vs=0.0
  endif
  rho=0.0
  seisp=0.0
  seiss=0.0
  snapvx=0.0
  snapvz=0.0

  if(lame)then
    call sep_read(lambda)
    call sep_read(mi,"mi")
    call sep_read(rho,"rho")
  else
    call sep_read(vp)
    call sep_read(vs,"vs")
    call sep_read(rho,"rho")
  endif

  ! Using the stability relation proposed by Graves (1996) in the work with
  ! surface waves
  !dt = 0.2*dx/4000.0

  !Moczo,et al (2000)
  !if(lame)then
  !  vmax=sqrt((maxval(lambda)-2.0*maxval(mi))/minval(rho))
  !else
  !  vmax=maxval(vp)
  !endif
  !dt=4.0/(7.0*sqrt(2.0))*(dx/vmax)

  nt = int(Tt/dt)

  ! calculate source length in time steps (making it 4 times the average
  ! distance between side lobes in a Ricker wavelet)
  nsource=4*nint(sqrt(6.0)/3.14159265/fp/dt)

  ! Write outputs to the terminal
  write(0,*)"============================================================="
  write(0,*)"BEGIN OF SUBROUTINE CALLS"
  write(0,*)"============================================================="
  write(0,*)"Values of the stability conditions:"
  write(0,*)"dx=dy=",dx
  write(0,*)"dt=",dt
  write(0,*)"source length=",nsource
  write(0,*)"seismogram length=",nseis
  write(0,*)"============================================================="

  ! Initialize the elastic modeling
  call elastic_init(nz,dz,nx,dx,nt,dt,xsource,zsource,nsource,fp,nsnap,nseis,sp,ti,abc,zrec)


  if(lame)then
    stat = elastic_op(lambda,mi,rho,seisp,seiss,snapvx,snapvz,lame)
  else
    stat = elastic_op(vp,vs,rho,seisp,seiss,snapvx,snapvz,lame)
  endif
  if (stat) then
    write(0,*) "ERROR ON ELASTIC_OP FUNCTION CALL ==> EXITING"
    stop
  endif

  ! Write outputs
  call sep_write(seisp)
  call sep_write(seiss,"seiss")

  call sep_write(snapvx,"snapvx")
  call sep_write(snapvz,"snapvz")

  call sep_close

  end_time = omp_get_wtime()

  ! Write outputs to the terminal
  write(0,*)"============================================================="
  write(0,*)"TOTAL ELAPSED TIME"
  write(0,*)"============================================================="
  write(0,*)"Total=",end_time - start_time, "seconds."
  write(0,*)"============================================================="

end program
