! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! FD elastic operator module
! This is the main subroutine that calls
! the FD derivatives, sources, etc.
module elastic_mod_new

  use source_mod_new
  use seis_mod_new
  use elasticlib_mod_new
  use snap_mod_new
  use boundary_mod_new

  implicit none
  integer, private :: nt, nx, nz, sp, ti, abc, zrec, ix
  integer, private :: xsource, zsource, nsource, nsnap, nseis
  real, private    :: dt, dx, dz, fp, source

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine elastic_init(nz_in,dz_in,nx_in,dx_in,nt_in,dt_in,xsource_in,  &
                          zsource_in,nsource_in, fp_in, nsnap_in, nseis_in,&
                          sp_in, ti_in, abc_in, zrec_in)

    integer             :: nz_in,nx_in,nt_in,xsource_in,zsource_in,nsource_in
    integer             :: nsnap_in, nseis_in, sp_in, ti_in, abc_in, zrec_in
    real                :: dz_in,dx_in,dt_in, fp_in

    nz = nz_in
    dz = dz_in
    nt = nt_in
    dt = dt_in
    nx = nx_in
    dx = dx_in
    fp = fp_in
    xsource = xsource_in
    zsource = zsource_in
    nsource = nsource_in
    nsnap   = nsnap_in
    nseis = nseis_in
    sp = sp_in
    ti = ti_in
    abc = abc_in
    zrec = zrec_in

  end subroutine

  ! Function that is called by Main
  function elastic_op(vp, vs, rho, seisp, seiss, snapvx, snapvz,lame) result(stat)
    real,dimension(:,:)  :: vp, vs, rho, seisp, seiss
    real,dimension(:,:,:):: snapvx, snapvz
    integer              :: stat
    logical,intent(in),optional :: lame

    stat=1
    call elastic_op2(vp, vs, rho, seisp, seiss, snapvx, snapvz,lame)
    stat=0
  end function

  ! Subroutine used by function
  subroutine elastic_op2(vp, vs, rho, seisp, seiss, snapvx, snapvz,lame)
    integer                             :: it, ix, iz, dsnap, dseis
    integer                             :: countsnap,countseis, temp
    real, dimension(nz,nx)              :: vp,vs,rho
    real, dimension(nseis,nx-2*abc)     :: seisp, seiss
    real, dimension(nz,nx,nsnap)        :: snapvx,snapvz

    real, allocatable, dimension(:,:)   :: mi, mi2, lambda
    real, allocatable, dimension(:,:)   :: Vx,Vz,sigmaxx,sigmazz,sigmaxz
    real, allocatable, dimension(:,:)   :: dxsigma,dzsigma,exx, ezz, exz
    real, allocatable, dimension(:,:)   :: rhox, rhoz
    real                                :: aux1, t, nsource2

    integer                             :: stat
    logical,intent(in),optional         :: lame

    !Memory allocation

    !Allocate lame parameters
    allocate(mi(nz,nx))
    allocate(mi2(nz,nx))
    allocate(lambda(nz,nx))
    allocate(rhox(nz,nx))
    allocate(rhoz(nz,nx))

    !Allocate Velocities and stresses
    allocate(Vx(nz,nx))
    allocate(Vz(nz,nx))
    allocate(sigmaxx(nz,nx))
    allocate(sigmazz(nz,nx))
    allocate(sigmaxz(nz,nx))

    !Allocate stress derivatives
    allocate(dxsigma(nz,nx))
    allocate(dzsigma(nz,nx))

    !Allocate strains
    allocate(exx(nz,nx))
    allocate(ezz(nz,nx))
    allocate(exz(nz,nx))

    !Initialize values
    mi     =0.0
    mi2    =0.0
    lambda =0.0
    Vx     =0.0
    Vz     =0.0
    sigmaxx=0.0
    sigmazz=0.0
    sigmaxz=0.0
    dxsigma=0.0
    dzsigma=0.0
    rhox   =0.0
    rhoz   =0.0
    exx    =0.0
    ezz    =0.0
    exz    =0.0
    dseis = (nt/nseis)+1
    dsnap = nt/nsnap
    countsnap = 1
    countseis = 1
    nsource2=2*nint(sqrt(6.0)/3.14159265/fp/dt)

    !Define Lame parameters
    aux1=(dt/dx)
    do ix=1,nx-1
      do iz=1,nz-1
        if(lame)then
          lambda(iz,ix)=vp(iz,ix)
          mi(iz,ix)=vs(iz,ix)
        else
          mi(iz,ix)=vs(iz,ix)*vs(iz,ix)*rho(iz,ix)
          lambda(iz,ix)=vp(iz,ix)*vp(iz,ix)*rho(iz,ix)-2.0*mi(iz,ix)
        endif
        rhox(iz,ix)=aux1*0.5*((1.0/rho(iz,ix+1)) + (1.0/rho(iz,ix)))
        rhoz(iz,ix)=aux1*0.5*((1.0/rho(iz+1,ix)) + (1.0/rho(iz,ix)))
      enddo
    enddo
    mi2(1,1)=mi(1,1)
    do iz=2,nz
      mi2(iz,1)=0.5*(mi(iz-1,ix)+mi(iz,ix))
    enddo
    do ix=2,nx
      do iz=2,nz
        !Introducing effective medium parameters (essentially correcting the
        !value of mi for the position of exz)
        mi2(iz,ix)=0.25*(mi(iz-1,ix-1)+mi(iz-1,ix)+mi(iz,ix-1)+mi(iz,ix))
      enddo
    enddo

    !Initialize subroutines
    call source_init(fp,dt,dx,nsource,nz,nx)
    call seis_init(nz,nx, dx, abc, zrec, seisp, seiss)
    call elasticmod_init(nz,nx,sp)
    call boundary_init(nz,nx,abc)
    call snap_init(nz,nx,snapvx,snapvz)
    
    !Time loop for 2nd order time
    write(0,*)"============================================================="
    write(0,*)"BEGIN OF TIME MODELING"
    write(0,*)"============================================================="
    do it=1,nt
      if (it<nsource) then
        call source_op2(it, source)
      else
        source=0.0
      endif

      !Particle velocities
      stat = dx_op(model=dxsigma,data=sigmaxx)
      stat = dz_op3(model=dzsigma,data=sigmaxz)
      Vx = Vx + rhox*(dxsigma+dzsigma)
          
      stat = dx_op3(model=dxsigma,data=sigmaxz)
      stat = dz_op(model=dzsigma,data=sigmazz)
      Vz = Vz + rhoz*(dxsigma+dzsigma)

      !Introduce absorbing boundary conditions to velocity
      stat = boundary_op(model=Vx,data=Vx)
      stat = boundary_op(model=Vz,data=Vz)

      !Strain components
      stat = dx_op3(model=exx,data=Vx)
      stat = dz_op3(model=ezz,data=Vz)
      stat = dxz_op(model=exz,data1=Vx, data2=Vz)

      !Stress components
      sigmaxx=sigmaxx+aux1*((lambda+2*mi)*exx+lambda*ezz)
      sigmazz=sigmazz+aux1*((lambda+2*mi)*ezz+lambda*exx)
      sigmaxz=sigmaxz+aux1*(mi2*(exz))

      !ntroduce absorbing boundary conditions to stress
      stat = boundary_op(model=sigmaxx,data=sigmaxx)
      stat = boundary_op(model=sigmazz,data=sigmazz)
      stat = boundary_op(model=sigmaxz,data=sigmaxz)

      !Explosive source positioning
      source=100.0*source
      call explosive_source(source,xsource,zsource,sigmaxx,sigmazz)
      !call shear_source(source,xsource,zsource,sigmaxx,sigmazz)

      !Write seismograms
      if (it.gt.nsource2.and.mod(it,dseis)==0) then
        call seis_op2(countseis, Vx, Vz)
        countseis = countseis + 1
        write(0,*)"Seismogram time sample"
      endif

      !Write snapshots
      if (mod(it,dsnap)==0.and.countsnap.le.nsnap) then
        call snap_op2(countsnap,Vx,Vz)
        countsnap = countsnap + 1
      endif

      !Write to terminal
      write(0,*)"Time step is:",it," of ",nt
    enddo

    write(0,*)"Last seismogram sample is:",countseis

  end subroutine

end module
